@@ -181,20 +181,28 @@ static struct security_descriptor *descr_handle_sd_flags(TALLOC_CTX *mem_ctx,
 	final_sd->type = SEC_DESC_SELF_RELATIVE;
 
 	if (sd_flags & (SECINFO_OWNER)) {
-		final_sd->owner_sid = talloc_memdup(mem_ctx, new_sd->owner_sid, sizeof(struct dom_sid));
+		if (new_sd->owner_sid) {
+			final_sd->owner_sid = talloc_memdup(mem_ctx, new_sd->owner_sid, sizeof(struct dom_sid));
+		}
 		final_sd->type |= new_sd->type & SEC_DESC_OWNER_DEFAULTED;
 	}
 	else if (old_sd) {
-		final_sd->owner_sid = talloc_memdup(mem_ctx, old_sd->owner_sid, sizeof(struct dom_sid));
+		if (old_sd->owner_sid) {
+			final_sd->owner_sid = talloc_memdup(mem_ctx, old_sd->owner_sid, sizeof(struct dom_sid));
+		}
 		final_sd->type |= old_sd->type & SEC_DESC_OWNER_DEFAULTED;
 	}
 
 	if (sd_flags & (SECINFO_GROUP)) {
-		final_sd->group_sid = talloc_memdup(mem_ctx, new_sd->group_sid, sizeof(struct dom_sid));
+		if (new_sd->group_sid) {
+			final_sd->group_sid = talloc_memdup(mem_ctx, new_sd->group_sid, sizeof(struct dom_sid));
+		}
 		final_sd->type |= new_sd->type & SEC_DESC_GROUP_DEFAULTED;
 	} 
 	else if (old_sd) {
-		final_sd->group_sid = talloc_memdup(mem_ctx, old_sd->group_sid, sizeof(struct dom_sid));
+		if (old_sd->group_sid) {
+			final_sd->group_sid = talloc_memdup(mem_ctx, old_sd->group_sid, sizeof(struct dom_sid));
+		}
 		final_sd->type |= old_sd->type & SEC_DESC_GROUP_DEFAULTED;
 	}
 
@@ -248,7 +256,7 @@ static DATA_BLOB *get_new_descriptor(struct ldb_module *module,
 	enum ndr_err_code ndr_err;
 	struct ldb_context *ldb = ldb_module_get_ctx(module);
 	struct auth_session_info *session_info
-		= ldb_get_opaque(ldb, "sessionInfo");
+		= ldb_get_opaque(ldb, DSDB_SESSION_INFO);
 	const struct dom_sid *domain_sid = samdb_domain_sid(ldb);
 	char *sddl_sd;
 	struct dom_sid *default_owner;
@@ -408,7 +416,7 @@ static DATA_BLOB *get_new_descriptor(struct ldb_module *module,
 	}
 
 	sddl_sd = sddl_encode(mem_ctx, final_sd, domain_sid);
-	DEBUG(10, ("Object %s created with desriptor %s\n\n", ldb_dn_get_linearized(dn), sddl_sd));
+	DEBUG(10, ("Object %s created with descriptor %s\n\n", ldb_dn_get_linearized(dn), sddl_sd));
 
 	linear_sd = talloc(mem_ctx, DATA_BLOB);
 	if (!linear_sd) {
@@ -493,7 +501,7 @@ static int descriptor_search_callback(struct ldb_request *req, struct ldb_reply
 	struct ldb_val *sd_val = NULL;
 	struct ldb_message_element *sd_el;
 	DATA_BLOB *show_sd;
-	int ret;
+	int ret = LDB_SUCCESS;
 
 	ac = talloc_get_type(req->context, struct descriptor_context);
 
@@ -643,7 +651,7 @@ static int descriptor_add(struct ldb_module *module, struct ldb_request *req)
 	 * The SD_FLAG control is ignored on add
 	 * and we default to all bits set.
 	 */
-	sd_flags = 0xF;
+	sd_flags = SECINFO_OWNER|SECINFO_GROUP|SECINFO_SACL|SECINFO_DACL;
 
 	sd = get_new_descriptor(module, dn, req,
 				objectclass, parent_sd,
@@ -1041,9 +1049,9 @@ static int descriptor_extended_sec_desc_propagation(struct ldb_module *module,
 	}
 
 	if (parent_change != NULL) {
-		DLIST_ADD_END(parent_change->children, c, NULL);
+		DLIST_ADD_END(parent_change->children, c);
 	} else {
-		DLIST_ADD_END(descriptor_private->changes, c, NULL);
+		DLIST_ADD_END(descriptor_private->changes, c);
 	}
 
 	return ldb_module_done(req, NULL, NULL, LDB_SUCCESS);
@@ -1136,7 +1144,11 @@ static int descriptor_sd_propagation_object(struct ldb_module *module,
 		ret = ldb_wait(sub_req->handle, LDB_WAIT_ALL);
 	}
 	if (ret != LDB_SUCCESS) {
-		return ldb_module_operr(module);
+		ldb_asprintf_errstring(ldb_module_get_ctx(module),
+				       "descriptor_modify on %s failed: %s",
+				       ldb_dn_get_linearized(msg->dn),
+				       ldb_errstring(ldb_module_get_ctx(module)));
+		return LDB_ERR_OPERATIONS_ERROR;
 	}
 
 	if (sd_propagation_control->critical != 0) {
@@ -1178,8 +1190,39 @@ static int descriptor_sd_propagation_recursive(struct ldb_module *module,
 	const char * const no_attrs[] = { "@__NONE__", NULL };
 	struct descriptor_changes *c;
 	struct descriptor_changes *stopped_stack = NULL;
+	enum ldb_scope scope;
 	int ret;
 
+	/*
+	 * First confirm this object has children, or exists (depending on change->force_self)
+	 * 
+	 * LDB_SCOPE_SUBTREE searches are expensive.
+	 *
+	 * Note: that we do not search for deleted/recycled objects
+	 */
+	ret = dsdb_module_search(module,
+				 change,
+				 &res,
+				 change->dn,
+				 LDB_SCOPE_ONELEVEL,
+				 no_attrs,
+				 DSDB_FLAG_NEXT_MODULE |
+				 DSDB_FLAG_AS_SYSTEM,
+				 NULL, /* parent_req */
+				 "(objectClass=*)");
+	if (ret != LDB_SUCCESS) {
+		return ret;
+	}
+
+	if (res->count == 0 && !change->force_self) {
+		TALLOC_FREE(res);
+		return LDB_SUCCESS;
+	} else if (res->count == 0 && change->force_self) {
+		scope = LDB_SCOPE_BASE;
+	} else {
+		scope = LDB_SCOPE_SUBTREE;
+	}
+
 	/*
 	 * Note: that we do not search for deleted/recycled objects
 	 */
@@ -1187,7 +1230,7 @@ static int descriptor_sd_propagation_recursive(struct ldb_module *module,
 				 change,
 				 &res,
 				 change->dn,
-				 LDB_SCOPE_SUBTREE,
+				 scope,
 				 no_attrs,
 				 DSDB_FLAG_NEXT_MODULE |
 				 DSDB_FLAG_AS_SYSTEM,
